@@ -26,6 +26,7 @@
 
 #include <iostream>
 #include <string>
+#include <signal.h>
 
 #include "libnormaliz/version.h"
 
@@ -33,32 +34,59 @@ namespace libnormaliz {
 
 namespace Type {
 enum InputType {
-    integral_closure,
-    polyhedron,
-    normalization,
+    //
+    // homogeneous generators
+    //
     polytope,
     rees_algebra,
-    inequalities,
-    strict_inequalities,
-    signs,
-    strict_signs,
-    equations,
-    congruences,
-    inhom_inequalities,
-    dehomogenization,
-    inhom_equations,
-    inhom_congruences,
-    lattice_ideal,
-    grading,
-    excluded_faces,
+    subspace,
+    cone,
+    cone_and_lattice,
     lattice,
     saturation,
-    cone,
-    offset,
+    //
+    // inhomogeneous generators
+    //
     vertices,
+    offset,
+    //
+    // homogeneous constraints
+    //
+    inequalities,
+    signs,
+    equations,
+    congruences,
+    //
+    // inhomogeneous constraints
+    //
+    inhom_equations,
+    inhom_inequalities,
+    strict_inequalities,
+    strict_signs,
+    inhom_congruences,
+    //
+    // linearforms
+    //
+    grading,
+    dehomogenization,
+    //
+    // special
+    open_facets,
+    projection_coordinates,
+    excluded_faces,
+    lattice_ideal, 
+    //
+    // prwecomputed data
+    //
     support_hyperplanes,
-    cone_and_lattice,
-    subspace
+    extreme_rays,
+    // hilbert_basis_rec_cone,
+    //
+    // deprecated
+    //
+    integral_closure,
+    normalization,
+    polyhedron
 };
 } //end namespace Type
 
@@ -81,16 +109,16 @@ typedef unsigned int key_t;
 extern bool verbose;
 extern size_t GMP_mat, GMP_hyp, GMP_scal_prod;
 extern size_t TotDet;
-
 /*
  * If this variable is set to true, the current computation is interrupted and
  * an InterruptException is raised.
  */
-extern bool nmz_interrupted;
+extern volatile sig_atomic_t nmz_interrupted;
+
+extern bool nmz_scip; // controls the use of Scip
 
 #define INTERRUPT_COMPUTATION_BY_EXCEPTION \
 if(nmz_interrupted){ \
-    nmz_interrupted = false; \
     throw InterruptException( "external interrupt" ); \
 }
 
@@ -99,6 +127,11 @@ if(nmz_interrupted){ \
 // extern bool test_arithmetic_overflow;
 // extern long overflow_test_modulus;
 
+extern long default_thread_limit;
+extern long thread_limit;
+extern bool parallelization_set;
+long set_thread_limit(long t);
+
 /* set the verbose default value */
 bool setVerboseDefault(bool v);
 /* methods to set and use the output streams */
