@@ -35,6 +35,8 @@
 #include <libnormaliz/libnormaliz.h>
 #include <libnormaliz/integer.h>
 #include <libnormaliz/convert.h>
+#include <libnormaliz/vector_operations.h>
+// #include <libnormaliz/sublattice_representation.h>
 
 //---------------------------------------------------------------------------
 
@@ -43,10 +45,12 @@ using std::list;
 using std::vector;
 using std::string;
 
+template<typename Integer> class Sublattice_Representation;
+
 template<typename Integer> class Matrix {
 
     template<typename> friend class Matrix;
-    template<typename> friend class Lineare_Transformation;
+    // template<typename> friend class Lineare_Transformation;
     template<typename> friend class Sublattice_Representation;
     
     // public:
@@ -116,8 +120,7 @@ template<typename Integer> class Matrix {
     
     bool SmithNormalForm_inner(size_t& rk, Matrix<Integer>& Right);
     
-    vector<Integer> optimal_subdivision_point_inner() const;
-    
+    vector<Integer> optimal_subdivision_point_inner() const;  
 
 //---------------------------------------------------------------------------
 //                      Pivots for rows/columns operations
@@ -127,11 +130,11 @@ template<typename Integer> class Matrix {
     //0<abs(x)<=abs(y) for all y!=0 in the right-lower submatrix of this
     //described by an int corner
 
-    long pivot_column(size_t col);  //Find the position of an element x with
+    long pivot_in_column(size_t col);  //Find the position of an element x with
     //0<abs(x)<=abs(y) for all y!=0 in the lower half of the column of this
     //described by an int col
     
-    long pivot_column(size_t row,size_t col); //in column col starting from row
+    long pivot_in_column(size_t row,size_t col); //in column col starting from row
     
 //---------------------------------------------------------------------------
 //                     Helpers for linear systems
@@ -168,6 +171,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     Matrix(size_t row, size_t col, Integer value); //constructor, all entries set to value
     Matrix(const vector< vector<Integer> >& elem); //constuctor, elem=elem
     Matrix(const list< vector<Integer> >& elems);
+    Matrix(const vector<Integer>& row);
 
 //---------------------------------------------------------------------------
 //                             Data access
@@ -176,7 +180,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     void write_column(size_t col, const vector<Integer>& data); //write a column
     void print(const string& name, const string& suffix) const;         //  writes matrix into name.suffix
     void print_append(const string& name,const string& suffix) const;  // the same, but appends matrix
-    void print(std::ostream& out, bool with_format=false) const;          // writes matrix to the stream
+    void print(std::ostream& out, bool with_format=true) const;          // writes matrix to the stream
     void pretty_print(std::ostream& out, bool with_row_nr=false) const;  // writes matrix in a nice format to the stream                   // read a row
     size_t nr_of_rows() const;                       // returns nr
     size_t nr_of_columns() const;                   // returns nc
@@ -191,6 +195,9 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     Matrix submatrix(const vector<key_t>& rows) const;
     Matrix submatrix(const vector<int>& rows) const;
     Matrix submatrix(const vector<bool>& rows) const;
+    
+    Matrix select_columns(const vector<bool>& cols) const;
+    Matrix selected_columns_first(const vector<bool>& cols) const;
 
     void swap (Matrix<Integer>& x);
 
@@ -205,9 +212,10 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     Matrix& remove_zero_rows(); // remove zero rows, modifies this
 
     // resizes the matrix to the given number of rows/columns
-    // if the size shrinks it will keep all its allocated memory
+    // if only the number of rows shrinks it will keep all its allocated memory
+    // but with delete_rows=true they will really be deleted 
     // useful when the size varies
-    void resize(size_t nr_rows);
+    void resize(size_t nr_rows, bool delete_rows=false);
     void resize(size_t nr_rows, size_t nr_cols);
     void resize_columns(size_t nr_cols);
     void Shrink_nr_rows(size_t new_nr_rows);
@@ -233,6 +241,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
         return elem[row][col];
     }
     inline const vector< vector<Integer> >& get_elements() const {
+        assert(nr==elem.size());
         return elem;
     }
     inline vector<Integer> const& operator[] (size_t row) const {
@@ -247,6 +256,11 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     void set_nr(size_t rows){
         nc=rows;
     }
+    
+    //  convert the remaining matrix to nmz_float
+    Matrix<nmz_float> nmz_float_without_first_column() const;
+    
+    
 
 //---------------------------------------------------------------------------
 //                  Basic matrices operations
@@ -256,7 +270,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     Matrix multiplication(const Matrix& A) const;          // returns this*A
     Matrix multiplication(const Matrix& A, long m) const;// returns this*A (mod m)
     bool equal(const Matrix& A) const;             // returns this==A
-    bool equal(const Matrix& A, long m) const;     // returns this==A (mod m)
+    // bool equal(const Matrix& A, long m) const;     // returns this==A (mod m)
     Matrix transpose() const;                     // returns the transpose of this
     
     bool is_diagonal() const;
@@ -371,7 +385,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
                     
 // homogenous linear systems
 
-    Matrix<Integer> kernel () const;
+    Matrix<Integer> kernel (bool use_LLL=true) const;
     // computes a ZZ-basis of the solutions of (*this)x=0
     // the basis is formed by the ROWS of the returned matrix
                     
@@ -421,7 +435,7 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
     
     Matrix<Integer> solve_congruences(bool& zero_modulus) const;
     
-// saturate sublattice
+// saturate sublattice generated by rows
     
     void saturate();
 
@@ -436,9 +450,106 @@ size_t row_echelon_inner_bareiss(bool& success, Integer& det);
 // find an inner point in the cone spanned by the rows of the matrix
     
     vector<Integer> find_inner_point();
+
+//  LLL (for functions that return transformation matrices see below)
+    
+    Matrix<Integer> LLL() const;
+    
+    Matrix<Integer> LLL_transpose() const;
+    
+    void GramSchmidt(Matrix<nmz_float>& B, Matrix<nmz_float>& M, int from, int to);
+
 };
 //class end *****************************************************************
 
+//---------------------------------------------------------------------------
+//                  LLL with returned transformation matrices
+//---------------------------------------------------------------------------
+
+template<typename Integer> // to break circular dependence
+void v_el_trans(const vector<Integer>& av,vector<Integer>& bv, const Integer& F, const size_t start);
+
+template<typename Integer, typename Number> // ditto
+Matrix<Number> LLL_red(const Matrix<Number>& U, Matrix<Integer>& T, Matrix<Integer>& Tinv){
+// returns Lred =LLL_reduced(M) (sublattice generated by the rows!)
+// Lred=T*M, Tinv=inverse(T)
+// Original version with c = 0.9
+    
+    T=Tinv=Matrix<Integer>(U.nr_of_rows());
+    
+    Matrix<Number> Lred=U;
+    size_t dim=U.nr_of_columns();
+    int n=U.nr_of_rows();
+    // pretty_print(cout);
+    assert((int) U.rank()==n);
+    if(n<=1)
+        return Lred;
+    
+    Matrix<nmz_float> G(n,dim);
+    Matrix<nmz_float> M(n,n);
+    
+    Lred.GramSchmidt(G,M,0,2);
+    
+    int i=1;
+    while(true){
+        
+        for(int j=i-1;j>=0;--j){
+            Integer fact;
+            /* cout << "MMMMM " << i << " " << j << " " << M[i][j] << endl;
+            cout << i << "---" << G[i];
+            cout << j << "---" << G[j];*/
+            convert(fact,round(M[i][j]));
+            // cout << fact << " " << M[i][j] << endl;
+            if(fact!=0){
+                v_el_trans<Number>(Lred[j],Lred[i],-convertTo<Number>(fact),0);
+                v_el_trans<Integer>(T[j],T[i],-fact,0);
+                v_el_trans<Integer>(Tinv[i],Tinv[j],fact,0);
+                Lred.GramSchmidt(G,M,i,i+1); 
+            }
+        }
+        if(i==0){
+            i=1;
+            Lred.GramSchmidt(G,M,0,2);
+            continue;
+        }
+        nmz_float t1=v_scalar_product(G[i-1],G[i-1]);
+        nmz_float t2=v_scalar_product(G[i],G[i]);
+        nmz_float fact=0.9-M[i][i-1]*M[i][i-1];
+        if(t2<fact*t1){
+            std::swap(Lred[i],Lred[i-1]);
+            std::swap(T[i],T[i-1]);
+            std::swap(Tinv[i],Tinv[i-1]);
+            Lred.GramSchmidt(G,M,i-1,i); // i-1,i+1);
+            // cout << i-1 << "---" << G[i-1];
+            i--;
+        }
+        else{
+            i++;
+            if(i>=n)
+                break;
+            Lred.GramSchmidt(G,M,i,i+1);
+        }
+    }
+    
+    Tinv=Tinv.transpose();
+    
+    return Lred;
+}
+
+template<typename Integer, typename  Number>
+Matrix<Number> LLL_red_transpose(const Matrix<Number>& U, Matrix<Integer>& T, Matrix<Integer>& Tinv){
+// column version -- needed for coordinate transformations in ambient lattice
+// returns Lred=this*T, Tinv=inverse(T)    
+    
+    Matrix<Number> this_trans=U.transpose();
+    Matrix<Number> red_trans;
+    Matrix<Integer> T_trans, Tinv_trans;
+    red_trans=LLL_red(this_trans,T_trans,Tinv_trans);
+    T=T_trans.transpose();
+    Tinv=Tinv_trans.transpose();
+    return red_trans.transpose();
+}
+
 //---------------------------------------------------------------------------
 //                  Utilities
 //---------------------------------------------------------------------------
@@ -482,6 +593,30 @@ void mpz_submatrix(Matrix<mpz_class>& sub, const Matrix<Integer>& mother, const
 template<typename Integer>
 void mpz_submatrix_trans(Matrix<mpz_class>& sub, const Matrix<Integer>& mother, const vector<key_t>& selection);
 
+template<typename ToType, typename FromType>
+void convert(Matrix<ToType>& to_mat, const Matrix<FromType>& from_mat){
+    size_t nrows = from_mat.nr_of_rows();
+    size_t ncols = from_mat.nr_of_columns();
+    to_mat.resize(nrows, ncols);
+    for(size_t i=0; i<nrows; ++i)
+        for(size_t j=0; j<ncols; ++j)
+            convert(to_mat[i][j], from_mat[i][j]);
+}
+
+//---------------------------------------------------------------------------
+//                  Matrix relateed functions
+//---------------------------------------------------------------------------
+// determines the maximal subsets in a vector of subsets given by their indicator vectors
+// result returned in is_max_subset -- must be initialized outside
+// only set to false in this routine
+// if a set occurs more than once, only the last instance is recognized as maximal
+template<typename IncidenceVector>
+void maximal_subsets(const vector<IncidenceVector>& ind, vector<bool>& is_max_subset);
+
+
+
+
+
 } // namespace
 
 //---------------------------------------------------------------------------
